//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

const BYTE level_table[10] = { 0, 1, 2, 4, 8, 16, 32, 64, 128, 255 };

BYTE g_Speed;
BYTE g_SpeedCount;
BYTE g_int;

/* rolling anime */
BYTE g_Position;
BYTE g_StepCount;
BYTE g_Fade[8];
BYTE g_FadeTiming;

/* ------------------------------------- */
/* --- PWM1 interrupt */
#pragma interrupt_handler Timer1Int
void Timer1Int(void)
{
	BYTE in = PRT1DR;
	
	if ((in & 0b10000000) == 0) {
		if (g_Speed < 16) {
			if (g_Speed == 0) {
				g_Speed = 4;
			}
			else {
				g_Speed += 1;
			}
		}
	}
	else {
		g_SpeedCount += 1;
		if (g_SpeedCount > 4) {
			if (g_Speed > 0) {
				g_Speed -= 1;
				if (g_Speed == 0) {
					g_Position = (g_Position + 1) % 8;
				}
			}
			g_SpeedCount = 0;
		}
	}
	g_int = 0;
}

void Timer1Wait(void) {
	g_int = 1;
	while(g_int) {}
}

/* ------------------------------------- */
/* --- LED  */
void led_set(BYTE port, BYTE level)
{
	BYTE s, rs;
	
	s = 0b00000001 << port;
	rs = s ^ 0xff;
	if (level == 255) {
		//PRT0DM0 |= s;
		//PRT0DM1 |= s;
		//PRT0DM2 &= rs;
		PRT0DR &= rs;
		PRT0GS &= rs;
	}
	else if (level == 0) {
		//PRT0DM0 |= s;
		//PRT0DM1 |= s;
		//PRT0DM2 &= rs;
		PRT0DR |= s;
		PRT0GS &= rs;
	}
	else {
		level = 254 - level;
		switch(port) {
			case 0:
				PWM8_1_WritePulseWidth(level);
				break;
				
			case 1:
				PWM8_2_WritePulseWidth(level);
				break;
				
			case 2:
				PWM8_3_WritePulseWidth(level);
				break;
				
			case 3:
				PWM8_4_WritePulseWidth(level);
				break;
				
			case 4:
				PWM8_5_WritePulseWidth(level);
				break;
				
			case 5:
				PWM8_6_WritePulseWidth(level);
				break;
				
			case 6:
				PWM8_7_WritePulseWidth(level);
				break;
				
			case 7:
				PWM8_8_WritePulseWidth(level);
				break;
		}
		PRT0GS |= s;
	}
}

void light(BYTE port, BYTE pow)
{
	BYTE index;
	
	index = pow;
	if (index > 9) {
		index = 9;
	}
	led_set(port, level_table[index]);
}


void display(void)
{
	BYTE i;
	
	if (g_Fade[g_Position] < g_Speed) {
		g_Fade[g_Position] = g_Speed;
	}

	g_FadeTiming = (g_FadeTiming + 1) % 4;
	if (g_FadeTiming == 0) {
		for(i=0; i< 8; i++) {
			if (g_Fade[i] > 0) {
				g_Fade[i] -= 1;
			}
		}
	}
	
	for(i=0; i< 8; i++) {
		light(i, g_Fade[i]);
	}
}


/* ------------------------------------- */
/* --- main loop */
void main(void)
{
	int i;
    /* --- initialize */
	PWM8_1_Start();
	PWM8_2_Start();
	PWM8_3_Start();
	PWM8_4_Start();
	PWM8_5_Start();
	PWM8_6_Start();
	PWM8_7_Start();
	PWM8_8_Start();
	PWM8_1_EnableInt();
	M8C_EnableGInt;
	
	PRT1DR = 0b10000000;

	g_Position = 0;
	g_StepCount = 0;
	light(0, 0);
	light(1, 0);
	light(2, 0);
	light(3, 0);
	light(4, 0);
	light(5, 0);
	light(6, 0);
	light(7, 0);
	
	for(i=0; i<8; i++) {
		g_Fade[i] = 0;
	}
	g_FadeTiming = 0;

	while(1) {
		g_StepCount += g_Speed;
		if (g_StepCount > 0x60) {
			g_Position = (g_Position + 1) % 8;
			g_StepCount = 0;
		}
		display();
		// next
		Timer1Wait();
	}
}
